<?php


require_once(SITEROOT . "/class/DataObject.class.php");
require_once(SITEROOT . "/class/Parapheur.class.php");

class Log extends DataObject {
  protected $objectName = "logs";
  protected $prettyName = "Entre de journal";

  protected $date;
  protected $module;
  protected $severity;
  protected $issuer;
  protected $user_id;
  protected $visibility;
  protected $message;
  protected $timestamp;

  protected $dbFields = array( "date" => array( "descr" => "Date", "type" => "isDate", "mandatory" => true),
						 "module" => array( "descr" => "Module", "type" => "isString", "mandatory" => false),
						 "severity" => array( "descr" => "Svrit", "type" => "isString", "mandatory" => true),
						 "issuer" => array( "descr" => "metteur de l'entre", "type" => "isString", "mandatory" => false),
						 "user_id" => array( "descr" => "Identifiant de l'utilisateur", "type" => "isInt", "mandatory" => false),
						 "visibility" => array( "descr" => "Visibilit", "type" => "isString", "mandatory" => false),
						 "message" => array( "descr" => "Message", "type" => "isString", "mandatory" => true),
						 "timestamp" => array( "descr" => "Horodatage", "type" => "isString", "mandatory" => false),
	  						"authority_id" => array("descr"=>"Authority","type"=>"isInt","mandatory"=>false),
                            "authority_group_id" => array("descr"=>"Authority group","type"=>"isInt","mandatory"=>false),
                            "message_horodate" => array("descr"=>"Message horodate","type"=>"isString","mandatory"=>false),
						 );

  protected $severities = array( 0 => "DEBUG",
						   1 => "INFO",
						   2 => "WARNING",
						   3 => "ERROR",
						   4 => "CRITICAL"
						   );

  /**
   * \brief Constructeur d'une entre de log
   * \param id integer Numro d'id d'une collectivit existante avec lequel initialiser l'objet
   */
  public function __construct($id = false) {
    parent::__construct($id);
	if ($id) {
	  $this->init();
	}
  }

  /**
   * \brief Constructeur d'une entre de log  partir d'infos fournies en paramtres
   * \param $issuer chane : Crateur de l'entre de journal
   * \param $message chane : Message de l'entre de journal
   * \param $severity chane : Svrit du message
   * \param $date chane (optionnel) : Date de l'entre de journal (date courante par dfaut)
   * \param $visibility chane (optionnel) : Visibilit de l'entre de log ('USER', 'ADM' ou 'SADM') (vide par dfaut)
   * \param $module chane (optionnel) : Module concern par le message (vide par dfaut)
   * \param $user objet User (optionnel) : Utilisateur concern par l'entre de journal (vide par dfaut)
   * \return True en cas de succs, false sinon
   */
  public static function newEntry($issuer, $message, $severity, $date = false, $visibility = false, $module = false, $user = false ,$userid=false) {
	$logEntry = new Log();
	$logEntry->set("issuer", $issuer);
	$logEntry->set("message", $message);
	$logEntry->set("severity", $severity);
	if (! $date) {
	  $date = date('Y-m-d H:i:s');
	}

	$logEntry->set("date", $date);

	if ($module) {
	  $logEntry->set("module", $module);
	}


	if ($user) {
	    /** @var User $user */
		$userid = $user->getId();
	}

	if ($userid) {
		$logEntry->set("user_id",$userid);
	}
	if ($visibility) {
	  $logEntry->set("visibility", $visibility);
	}


      $sqlQuery = ObjectInstancierFactory::getObjetInstancier()->get("SQLQuery");

	  $authority_id = false;
	  $authority_group_id = false;
	  if ($userid) {
		  $userSQL = new UserSQL($sqlQuery);
		  $info = $userSQL->getInfo($userid);
		  if ($info) {
			  $authority_id = $info['authority_id'];

		  }
	  }
	  if ($authority_id){
		$authoritySQL = new AuthoritySQL($sqlQuery);
		$info = $authoritySQL->getInfo($authority_id);
		  if ($info){
			  $authority_group_id = $info['authority_group_id'];
		  }
	  }

	  $logEntry->set("authority_id",$authority_id);
	  $logEntry->set("authority_group_id",$authority_group_id);

	// Enregistrement de l'entre pour dterminer son id
	if (! $logEntry->save()) {
	  return false;
	}

	$message_horodate = $logEntry->generateMessageHorodate();

	if (! $timestamp = $logEntry->genTimestamp($message_horodate)) {
	  return false;
	}

	$logEntry->set("timestamp", $timestamp);
	$logEntry->set('message_horodate',$message_horodate);

	if (! $logEntry->save()) {
	  return false;
	}

	return true;
  }


  private function genTimestamp($data) {
	  if (TESTING_ENVIRONNEMENT){
		  //FIXME
		return "TESTING";
	  }
	// on cre un fichier contenant la concatnation de tous les champs de l'entre
	$logFile = tempnam('/tmp', 'tedetis_web_');



	if (! $this->writeLogEntryToFile($logFile, $data)) {
	  return false;
	}

	$parapheur = new Parapheur($data);
	$signature = $parapheur->getSignature();
	
	if (! $signature){
		$this->errorMsg = $parapheur->getLastError();
		return false;
	}

	  unlink($logFile);
	
	return $signature;
  } 

  /**
   * \brief Mthode qui dtermine si un utilisateur a la permission de visualiser l'entre de journal courante
   * \param $user User : Objet utilisateur concern
   */
  public function canView($user) {
	if ($user->isSuper()) {
	  return true;
	}

	if (! empty($this->user_id)) {
	  // L'utilisateur est "propritaire" de l'entre
	  if ($this->user_id == $user->getId()) {
		if ($user->isAdmin()) {
		  // Un admin peut voir tout ce qui n'est pas en visibilit SADM
		  if ($this->visibility != 'SADM') {
			return true;
		  }
		} else {
		  // Un utilisateur peut voir tout ce qui n'est pas en visibilit 'ADM', 'GADM' et 'SADM'
		  if ($this->visibility != 'SADM' && $this->visibility != 'ADM' && $this->visibility != 'GADM') {
			return true;
		  }
		}
	  } else {
		// Utilisateur non propritaire de l'entre
		// Si l'utilisateur est admin de collectivit, il peut voir l'entre si elle appartient  un utilisateur de sa collectivit
		$owner = new User($this->user_id);
		$owner->init();

		$hisAuthority = new Authority($owner->get("authority_id"));

		if (($user->isGroupAdmin() && $hisAuthority->isInGroup($user->get("authority_group_id"))) || ($user->isAuthorityAdmin() && $user->get("authority_id") == $owner->get("authority_id"))) {
		  return true;
		}
	  }
	} else {
	  // Entre non associ  un utilisateur
	  if ($user->isAuthorityAdmin() && ($this->visibility == 'ADM' || $this->visibility == 'USER')) {
		return true;
	  }

	  if ($user->isGroupAdmin() && ($this->visibility == 'GADM' || $this->visibility == 'ADM' || $this->visibility == 'USER')) {
		return true;
	  }

	  if (! $user->isAdmin() && $this->visibility == 'USER') {
		return true;
	  }
	}

	return false;
  }

  /**
   * \brief Mthode de gnration et d'envoi d'une archive contenant le fichier de l'entre de log et son horodatage
   * \return True en cas succs, false sinon
   */
  public function sendArchive() {
	if (isset($this->id) && ! empty($this->id)) {
	  $tmpDir = '/tmp/tedetis_web_export_timestamp';

	  for ($i = 0; $i <= 8; $i++) {
		$tmpDir .= rand(0, 9);
	  }

	  if (! @mkdir($tmpDir)) {
		$this->errorMsg = "Erreur systme de fichiers";
		return false;
	  }

	  $logFile = $tmpDir . "/tedetis_journal_" . $this->id . ".log";
	  $timestampFile = $logFile . ".sig";

	  if (! $this->writeLogEntryToFile($logFile, $this->retrieveMessageHorodate())) {
		return false;
	  }

	  if (! $this->writeTimestampToFile($timestampFile)) {
		return false;
	  }

	  if (! @chdir($tmpDir)) {
		$this->errorMsg =  "Erreur systme de fichiers";
		return false;
	  }

	  // Gnration de l'archive zip
	  $zipFile = "tedetis_journal_" . $this->id . ".zip";
	  $cmd = "/usr/bin/zip -9 " . $zipFile . " " . basename($logFile) . " " . basename($timestampFile);

	  exec($cmd, $out, $ret);

	  if ($ret != 0) {
		$return_status = false;
	  } else {
		// Envoi du fichier
		header("Content-type: application/zip");
		header('Content-disposition: attachment; filename="' . $zipFile . '"');
		// Celles-ci pour IE
		header("Expires: 0");
		header("Cache-Control: must-revalidate, post-check=0,pre-check=0");
		header("Pragma: public");

		if (! @readfile($tmpDir . "/" . $zipFile)) {
		  $this->errorMsg =  "Erreur d'envoi du fichier archive.";
		  $return_status = false;
		} else {
		  $return_status = true;
		}

		if (! Helpers::deleteFromFS($zipFile)) {
		  $this->errorMsg = "Erreur systme de fichiers";
		  return false;
		}
	  }

	  // Suprression des fichiers temporaires
	  // Bien laiss le rpertoire  la fin
	  if (! Helpers::deleteFromFS($logFile, $timestampFile, $tmpDir)) {
		$this->errorMsg =  "Erreur systme de fichiers";
		return false;
	  }

	  return $return_status;
	}

	return false;
  }

  /**
   * \brief Mthode d'criture de l'entre de journal dans un fichier
   * \return True en cas de succs, false sinon
   */
  public function writeLogEntryToFile($logFile, $data) {
	if (! file_put_contents($logFile, $data)) {
	  $this->errorMsg = "Erreur systme de fichiers.";
	  return false;
	}

	return true;
  }

  /**
   * Mthode d'obtention de l'entre de log en format concatn pour horodatage
   * @return string La chane de tous les champs spars par '**||**'
   */
  public function generateMessageHorodate() {
	$data[] = $this->id;
	$data[] = date('c', Helpers::getTimestampFromBDDDate($this->date));
	$data[] = $this->module;
	$data[] = $this->severity;
	$data[] = $this->issuer;
	$data[] = $this->user_id;
	$data[] = $this->visibility;
	$data[] = $this->message;


	$log = implode($data, "**||**");

	return $log;
  }

    /**
     *  Mthode d'obtention de l'entre de log en format concatn pour horodatage
     * @return string La chane de tous les champs spars par '**||**'
     */
    public function retrieveMessageHorodate() {
        if ($this->get('message_horodate')){
            return $this->get('message_horodate');
        }
        //Ancienne mthode de gnration du message horodat
        $data[] = $this->id;
        $data[] = date('Y-m-d H:i:s', Helpers::getTimestampFromBDDDate($this->date));
        $data[] = $this->module;
        $data[] = $this->severity;
        $data[] = $this->issuer;
        $data[] = $this->user_id;
        $data[] = $this->visibility;
        $data[] = $this->message;


        $log = implode($data, "**||**");

        return $log;
    }


  /**
   * \brief Mthode d'criture de l'horodatage dans un fichier
   * \return True en cas de succs, false sinon
   */
  function writeTimestampToFile($timestampFile) {
	if (isset($this->timestamp) && ! empty($this->timestamp)) {
	  if (! file_put_contents($timestampFile, $this->timestamp)) {
		$this->errorMsg = "Erreur systme de fichiers.";
		return false;
	  }

	  return true;
	}

	return false;
  }

  /**********************/
  /* Mthodes statiques */
  /**********************/

  /**
   * \brief Mthode d'obtention d'une liste d'entres de journal
   * \param $cond (optionnel) chane Chane contenant les conditions (SQL)  appliquer  la fin de la requte BDD
   * \return Tableau des entres de journal
  */
  public function getLogEntriesList($cond = "") {
	if (! $this->pagerInit('logs.id, logs.date, logs.module, logs.severity, logs.issuer, logs.user_id, logs.message, logs.timestamp', 'logs LEFT JOIN users ON logs.user_id=users.id LEFT JOIN authorities ON users.authority_id=authorities.id', $cond)) {
	  return false;
	}

    return $this->data;
  }
}

