<?php


require_once(SITEROOT . "/class/DataObject.class.php");
require_once(SITEROOT . "/public.ssl/modules/actes/class/ActesEnvelope.class.php");



class ActesIncludedFile extends DataObject {
  
	protected $objectName = "actes_included_files";
  
  protected $envelope_id;
  protected $transaction_id;
  protected $filename;
  protected $posted_filename;
  protected $filetype;
  protected $filesize;
  protected $signature;

  protected $code_pj;


  protected $envelope;

  	private $tampon;
	private $date_affichage;


  
  
  protected $dbFields = array( "envelope_id" => array( "descr" => "Identifiant enveloppe", "type" => "isInt", "mandatory" => true),
						 "transaction_id" => array( "descr" => "Identifiant transaction", "type" => "isInt", "mandatory" => false),
						 "filename" => array( "descr" => "Nom du fichier mtier", "type" => "isString", "mandatory" => true),
						 "posted_filename" => array( "descr" => "Nom du fichier original", "type" => "isString", "mandatory" => false),
						 "filetype" => array( "descr" => "Type du fichier", "type" => "isString", "maxlength" => 499, "mandatory" => true),
						 "filesize" => array( "descr" => "Taille du fichier", "type" => "isInt", "mandatory" => true),
						 "signature" => array( "descr" => "Signature lectronique du fichier", "type" => "isString", "mandatory" => false),
	  					"code_pj" => array( "descr" => "Code de la PJ", "type" => "isString", "mandatory" => false)
						 );

  /**
   * \brief Constructeur
   * \param id integer Numro d'identifiant d'un fichier attach
   */
  public function __construct($id = false) {
	parent::__construct($id);

	if ($id) {
	  $this->init();
	  $this->initEnvelope();
	}
	
		$this->tampon = false;
	
  }

  /**
   * \brief Mthode d'initialisation de l'enveloppe contenant le fichier courant
   */
  public function initEnvelope() {
	if (isset($this->envelope_id)) {
	  $this->envelope = new ActesEnvelope($this->envelope_id);
	  $this->envelope->init();
	}
  }

  
  public function setTampon($date_affichage = false){
  	$this->tampon = true;
	  $this->date_affichage = $date_affichage;
  }
  
  public function sendSignature(){
  	
  }
  
  /**
   * \brief Mthode qui renvoie le fichier au navigateur
   */
  public function sendFile() {
	if (isset($this->filename)) {
	  // Il faut extraire le fichier demand dans un stockage temporaire
	  $tmpDir = "/tmp/" . Helpers::genTempName();

          $this->errorMsg='';
	  if (! @mkdir($tmpDir)) {
		$this->errorMsg .= "Erreur systme de fichiers";
		return false;
	  }
	  
	  if (! file_exists(ACTES_FILES_UPLOAD_ROOT . '/' . $this->envelope->get("file_path"))) {
		$this->errorMsg .= "Le fichier archive n'est pas/plus disponible.";
		return false;
	  }

	  $cmd = 'tar xzf ' . ACTES_FILES_UPLOAD_ROOT . '/' . $this->envelope->get("file_path") . " -C " . $tmpDir . " " . $this->filename;
	  
	  Trace::wrap_exec($cmd, $status, $ret);

	  $ret_value = true;

	  if ($status === false || $ret != 0) {
		$this->errorMsg .= "Erreur d'extraction du fichier demand (code " . $ret . ")";
		$ret_value = false;
	  } else {
		if (strlen($this->posted_filename)) {
		  $browserName = $this->posted_filename;
		} else {
		  $browserName = $this->filename;
		}

		$path_parts = pathinfo($this->filename);

		//FIXME SALE 
		if ($path_parts['extension'] == 'pdf' && $this->tampon){

			$objectInstancier = ObjectInstancierFactory::getObjetInstancier();

			$pathpdforig = $tmpDir . '/' .$this->filename;

			$acteTamponne = $objectInstancier->get("ActeTamponne");
			$acteTamponne->render($pathpdforig,$this->get("transaction_id"), $this->date_affichage);

		} elseif (! Helpers::sendFileToBrowser($tmpDir . "/" . $this->filename, $browserName, $this->filetype)) {
		  $this->errorMsg .= "Erreur envoi fichier";
		  $ret_value = false;
		}
 		  if (file_exists($tmpDir . "/" . $this->filename)){
				if (! unlink($tmpDir . "/" . $this->filename)) {
                        $this->errorMsg .= "Erreur suppression fichier";
                        $ret_value = false;
                    }
			}
	  }

	  // Suppression du rpertoire temporaire
          if (file_exists($tmpDir)){
            if (! rmdir($tmpDir)) {
		$this->errorMsg .= "Erreur suppression rpertoire temporaire";
		$ret_value = false;
            }
	  }//fin if test dossier $tmpDir

	  return $ret_value;
	}
  }
  
  function modificationPDF($pathpdforig, $pathpdfout){
      $cmdpdftk='timeout 10 pdftk '. $pathpdforig." stamp ".SITEROOT."/data-exemple/vide.pdf output ".$pathpdfout;
      Trace::wrap_exec($cmdpdftk, $status, $ret);
      if ($status === false || $ret != 0) {
          $cmdpdftk='timeout 10 pdfsam-console -f '. $pathpdforig ." -o ". $pathpdfout ." concat";
          Trace::wrap_exec($cmdpdftk, $status, $ret);
          if ($status === false || $ret != 0){
              $pathpdfout=$pathpdforig;
              //$this->errorMsg .= "Erreur lors de la convertion via pdftk et pdfsam";
          }
       }//fin if
      return $pathpdfout;
  }


  /**********************/
  /* Mthodes statiques */
  /**********************/

  /**
   * \brief Mthode de rcupration de la liste des fichiers associs  une transaction
   * \param id entier : identifiant de la transaction
   * \return Un tableau de description des fichiers
   */
  static public function fetchFilesList($id) {
	if (isset($id)) {
	  $sql = "SELECT id, filename AS name, posted_filename, filetype AS mimetype, filesize AS size, signature AS sign, code_pj FROM actes_included_files WHERE transaction_id=" . $id . " ORDER BY id";
	  
	  
	  $db =DatabasePool::getInstance();

	  $result = $db->select($sql);

	  $files = array();

	  if (! $result->isError()) {
		$files = $result->get_all_rows();
	  }
	}

	return $files;
  }
}
?>
