<?php
/*
 * TDTIS - Copyright 2006 Alternance-Soft
 * Contributeur : Jrme Schell, Aot 2006 
 *
 * contact@alternancesoft.com
 *
 * Ce logiciel est un programme informatique servant  la
 * dmatrialisation de l'administration. 
 *
 * Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilit au code source et des droits de copie,
 * de modification et de redistribution accords par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
 * seule une responsabilit restreinte pse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concdants successifs.
 *
 * A cet gard  l'attention de l'utilisateur est attire sur les risques
 * associs au chargement,   l'utilisation,   la modification et/ou au
 * dveloppement et  la reproduction du logiciel par l'utilisateur tant 
 * donn sa spcificit de logiciel libre, qui peut le rendre complexe  
 * manipuler et qui le rserve donc  des dveloppeurs et des professionnels
 * avertis possdant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invits  charger  et  tester  l'adquation  du
 * logiciel  leurs besoins dans des conditions permettant d'assurer la
 * scurit de leurs systmes et ou de leurs donnes et, plus gnralement, 
 *  l'utiliser et l'exploiter dans les mmes conditions de scurit. 
 *
 * Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accept les
 * termes.
*/
?>
<?php
/**
 * \class ActesClassification ActesClassification.class.php
 * \brief Cette classe permet de grer les classifications matires/sous-matires
 * \author Jrme Schell <j.schell@alternancesoft.com>
 * \date 03.08.2006
 * 
 *
 * Cette classe fournit des mthodes de gestion de la liste
 * de classification matres/sous-matires
 *
 * Modifications :
 * Auteur   Date       Commentaire
 *
 */

require_once(SITEROOT . "/class/DataObject.class.php");

class ActesClassification extends DataObject {
  protected $objectName = "actes_classification_requests";
  
  protected $request_date;
  protected $requested_by;
  protected $version_date;
  protected $xml_data;
  
  protected $dbFields = array( "request_date" => array( "descr" => "Date de la demande", "type" => "isDate", "mandatory" => true),
							   "requested_by" => array( "descr" => "Identifiant du demandeur", "type" => "isInt", "mandatory" => true),
							   "version_date" => array( "descr" => "Date de la version de la classification", "type" => "isDate", "mandatory" => false),
							   "xml_data" => array( "descr" => "Fichier XML de la classification", "type" => "isString", "mandatory" => false)
							   );

  /**
   * \brief Constructeur d'une requete de classification
   * \param id integer Numro d'identifiant d'une requete existante avec laquelle initialiser l'objet
   */
  public function __construct($id = false) {
	parent::__construct($id);
  }

  /**
   * \brief Mthode d'initialisation avec les donnes de la dernire requte en date ayant russie
   * \param $authority_id entier : Identifiant de la collectivit concerne
   * \return True en cas de succs, false sinon
   */
  public function initWithLastSuccessful($authority_id) {
 	if (is_numeric($authority_id)) {
	  $sql = "SELECT acr.id, acr.request_date, acr.requested_by, acr.version_date, acr.xml_data " . 
	  			" FROM actes_classification_requests acr " . 
	  			" LEFT JOIN users ON acr.requested_by=users.id " .
	  			" WHERE users.authority_id=" . $authority_id . " AND acr.xml_data IS NOT NULL " . 
	  			" ORDER BY request_date DESC, version_date DESC LIMIT 1";

	  $result = $this->db->select($sql);
	  
	  if (! $result->isError() && $result->num_row() == 1) {
		$row = $result->get_next_row();
		$this->request_date = Helpers::getFromBDD($row["request_date"]);
		$this->requested_by = Helpers::getFromBDD($row["requested_by"]);
		$this->version_date = Helpers::getFromBDD($row["version_date"]);
		$this->xml_data = Helpers::getFromBDD($row["xml_data"]);

		return true;
	  }
	}

	return false;
  }
  /**
   * \brief Mthode d'envoi de la classification
   * \return True en cas de succs, false sinon
   */
  public function pushXMLData() {
	if (! empty($this->xml_data)) {
	  header("Content-type: text/xml;charset=iso-8859-1");
	  header('Content-disposition: attachment; filename="classification.xml"');
	  // Celles-ci pour IE
	  header("Expires: 0");
	  header("Cache-Control: must-revalidate, post-check=0,pre-check=0");
	  header("Pragma: public");

	  echo $this->xml_data . "\r\n";
	} else {
	  return false;
	}

	return true;
  }


  /**********************/
  /* Mthodes statiques */
  /**********************/

  /**
   * \brief Mthode de vrification qu'une transaction de demande de classification n'existe pas dj dans la journe en cours
   * \param $authority_id entier : Identifiant de la collectivit concerne
   * \return True si une transaction a dj t faites dans la journe en cours, false sinon
  */
  public static function hasTodayRequest($authority_id) {
	if (is_numeric($authority_id)) {
	  $today = date('Y-m-d');
	  $sql = "SELECT DISTINCT at.id FROM actes_transactions at"
		. " LEFT JOIN actes_transactions_workflow atw ON at.id=atw.transaction_id"
		. " LEFT JOIN actes_envelopes ae ON at.envelope_id=ae.id"
		. " LEFT JOIN users ON ae.user_id=users.id"
		. " WHERE at.type='7' "
		// On ne tient compte que des demandes dans les tats post, en attente ou transmis (3), de ce fait si une demande tombe en erreur
		// il sera possible de refaire une requte dans la mme journe
		
		//Correger pour la report du bug: 190 =>lenteur du plateforme.
		. " AND (SELECT status_id FROM actes_transactions_workflow atw2 WHERE date = ( SELECT MAX(date) FROM actes_transactions_workflow atw3 WHERE atw3.transaction_id = atw2.transaction_id) AND transaction_id = at.id ORDER BY atw2.id DESC LIMIT 1) IN (1,2,3)"
		. " AND atw.date >= '" . $today . "' AND users.authority_id=" . $authority_id;

	  $db =DatabasePool::getInstance();

	  $result = $db->select($sql);
	  
	  if (! $result->isError() && $result->num_row() > 0) {
		return true;
	  }
	}

	return false;
  }


  /**
   * \brief Mthode d'obtention de la date de la dernire classification utilise
   * \param $authority_id entier : Identifiant de la collectivit concerne
   * \param $return_ansi bollen : si true (dfaut) retourne la date au format YYYY-MM-DD, format BDD sinon
   * \return La date de la dernire classification utilise
  */
  public static function getLastRevisionDate($authority_id, $return_ansi = true) {
	if (is_numeric($authority_id)) {
	  // TODO : voir s'il ne vaut mieux pas ordonner par ID et prendre la date de la dernire requte
	  $sql = "SELECT MAX(version_date) AS max_date FROM actes_classification_requests"
		. " LEFT JOIN users ON actes_classification_requests.requested_by=users.id"
		. " WHERE users.authority_id=" . $authority_id;

	  $db =DatabasePool::getInstance();

	  $result = $db->select($sql);
	  
	  if (! $result->isError() && $result->num_row() > 0) {
		$row = $result->get_next_row();
		if ($return_ansi) {
		  return Helpers::getANSIDateFromBDDDate($row["max_date"]);
		} else {
		  return $row["max_date"];
		}
	  }
	}

	return null;
  }

  /**
   * \brief Mthode de rcupration de la liste de classification
   * \param $authority_id entier : Identifiant de la collectivit concerne
   * \return Un tableau contenant la liste de classification matires/sous-matires
   */
  public static function getClassificationList($authority_id) {
	if (is_numeric($authority_id)) {
	  $sql = "SELECT id, level, code, parent_id, description FROM actes_classification_codes"
		. " WHERE authority_id=" . $authority_id . " ORDER BY level,code ASC";

	  $db =DatabasePool::getInstance();

	  $result = $db->select($sql);


	  $classif = array();

	  if (! $result->isError()) {
		while ($row = $result->get_next_row()) {
		  if (! isset($classif[$row["id"]])) {
			$classif[$row["id"]] = array();
		  }

		  $classif[$row["id"]]["id"] = $row["id"];
		  $classif[$row["id"]]["level"] = $row["level"];
		  $classif[$row["id"]]["code"] = $row["code"];
		  $classif[$row["id"]]["description"] = $row["description"];

		  if (isset($row["parent_id"]) && ! empty($row["parent_id"])) {
			$classif[$row["id"]]["parent_id"] = $row["parent_id"];

			if (! isset($classif[$row["parent_id"]])) {
			  $classif[$row["parent_id"]] = array();
			}

			$classif[$row["parent_id"]]["children_id"][] = $row["id"];
		  }
		}
	  } else {
		return false;
	  }

	  return $classif;
	}

	return false;
  }

  /**
   * \brief Mthode de rcupration du fichier XML de la classification

   * \return Une chane contenant les donnes XML de la classification
   */
  public static function getClassificationXMLData($authority_id) {

  }
}

